<?php

namespace Drupal\last_tweets;

use Abraham\TwitterOAuth\TwitterOAuth;

/**
 * Class LastTweets.
 *
 * @package Drupal\last_tweets
 *   Last twitter helper class.
 */
class LastTweetsHelper {

  /**
   * Get last tweets.
   *
   * @param mixed $twitter_limit
   *   Number of tweet limit.
   *
   * @return array
   *   Return renderable array.
   */
  public static function getLastTweets($twitter_limit = NULL) {

    $settings = \Drupal::service('config.factory')->getEditable('last_tweets.settings');
    $id = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $defaultLanguage = \Drupal::languageManager()->getDefaultLanguage()->getId();

    $use_for_all = $settings->get('use_for_all_' . $defaultLanguage);
    $id = ($use_for_all) ? $defaultLanguage : $id;
    $user_name = $settings->get('twitter_username_' . $id);
    $consumer_key = $settings->get('consumer_key_' . $id);
    $secret_key = $settings->get('secret_key_' . $id);
    $access_token = $settings->get('access_token_' . $id);
    $access_token_secret = $settings->get('access_token_secret_' . $id);

    if (!$twitter_limit) {
      $twitter_limit = $settings->get('twitter_limit');
    }

    // Create the connection.
    $twitter = new TwitterOAuth($consumer_key, $secret_key, $access_token, $access_token_secret);
    // Migrate over to SSL/TLS.
    $twitter->ssl_verifypeer = TRUE;
    // Load the Tweets.
    $tweets = $twitter->get('statuses/user_timeline', [
      'tweet_mode' => 'extended',
      'screen_name' => $user_name,
      'exclude_replies' => 'false',
      'include_rts' => 'true',
      'include_entities' => 'true',
      'count' => $twitter_limit,
    ]);

    // Output.
    foreach ($tweets as $t) {
      if (is_object($t)) {
        // Build tweet links.
        if (!empty($t->entities->urls)) {
          $tweetUrls = $t->entities->urls;
          foreach ($tweetUrls as $tweetUrl) {
            $tUrl = $tweetUrl->url;
            $t->full_text = str_replace($tUrl, '<a class="tweet-url" title="link tweet" href="' . $tUrl . '" >' . $tUrl . '</a>', $t->full_text);
          }
        }
        $textArr = explode(" ", $t->full_text);

        // Build hastags links.
        if (!empty($t->entities->hashtags)) {
          $hashtags = $t->entities->hashtags;
          foreach ($hashtags as $hashtag) {
            $tag = "#" . $hashtag->text;
            $tagLink = substr_replace($tag, '', 0, 1);
            foreach ($textArr as $key => $text) {
              $textArr[$key] = str_replace($tag, '<a class="hashtag" title="hashtag" href="https://twitter.com/hashtag/' . $tagLink . '?src=hash" >' . $tag . '</a>', $textArr[$key]);
            }
          }
        }

        // Build user mentions links.
        if (!empty($t->entities->user_mentions)) {
          $userMentions = $t->entities->user_mentions;
          foreach ($userMentions as $userMention) {
            $screenName = "@" . $userMention->screen_name;
            foreach ($textArr as $index => $text) {
              $screenNameLink = substr_replace($screenName, '', 0, 1);
              $title = t('mentionned user');
              $textArr[$index] = str_replace($screenName, '<a class="user-mentions" title=' . $title . ' href="https://twitter.com/' . $screenNameLink . '" >' . $screenName . '</a>', $textArr[$index]);
            }
          }
        }

        // Get tweet media.
        $medias = NULL;
        $t = property_exists($t, 'retweeted_status') ? $t->retweeted_status : $t;
        if (!empty($t->extended_entities->media)) {
          $medias = $t->extended_entities->media;
        }
        elseif (!empty($t->entities->media)) {
          $medias = $t->entities->media;
        }

        if (!empty($medias)) {
          $t->tweet_media = '';
          foreach ($medias as $media) {
            // Photo.
            if ($media->type == "photo") {
              $t->tweet_media .= '<a href="' . $media->url . '"><img alt="photo tweet" src = "' . $media->media_url_https . '"><a/>';
            }
            // Video.
            if ($media->type == "video") {
              $video_url = self::getVideoUrl($media->video_info->variants);
              if ($video_url) {
                $t->tweet_media .= '<video controls="" autoplay="" width="500px" height="auto" poster="' . $media->media_url_https . '"><source src="' . $video_url . '" type="video/mp4"></video>';
              }
            }
          }
        }
        $t->full_text = implode(" ", $textArr);
      }
    }

    // Create the HTML output.
    if (!empty($tweets)) {
      return [
        '#theme' => 'tweets_list',
        '#tweets' => $tweets,
        '#user_screen_name' => $user_name,
      ];
    }
    else {
      return [];
    }
  }

  /**
   * Video Url.
   *
   * @param array $variants
   *   Array of video variants from api.
   *
   * @return null|string
   *   Video url.
   */
  public static function getVideoUrl(array $variants) {
    if ($variants) {
      foreach ($variants as $variant) {
        if ($variant->content_type == 'video/mp4') {
          return $variant->url;
        }
      }
    }
    return NULL;
  }

}
